function [Hmat,fvec,Amat,bvec,Aeq,beq,x,dx,row,Aymat,fp_Aymat,Aymat_obs,fp_Aymat_obs,Amat_monotone] = ComputeMatrix(X,y,h,kernel,grid_point)

n = size(y,1);
% size returns a vector of length 2 with height and width of x
d = size(X,2); 
% Reassign d with the value of d(2) makes d a scalar with the value indicating the 
% number of input variables

%Equally spaced grid.
Num_grid = round(grid_point^(1/d));

for i = 1:d,
    x(:,i) = linspace(min(X(:,i))', max(X(:,i))', Num_grid);
    %Calculate distance of equally space grid
    dx(i) = (x(2,i)-x(1,i))*1.1;
end


if d == 2,
    [x1,x2]=ndgrid(x(:,1),x(:,2));
    x=[x1(:),x2(:)];
elseif d == 3,
    [x1,x2,x3]=ndgrid(x(:,1),x(:,2),x(:,3));
    x=[x1(:),x2(:),x3(:)];  
elseif d == 4,
    [x1,x2,x3,x4]=ndgrid(x(:,1),x(:,2),x(:,3),x(:,4));
    x=[x1(:),x2(:),x3(:),x4(:)];  
elseif d == 5,
    [x1,x2,x3,x4,x5]=ndgrid(x(:,1),x(:,2),x(:,3),x(:,4),x(:,5));
    x=[x1(:),x2(:),x3(:),x4(:),x5(:)];  
end

%Small x is point for evaluation
%Capital X is observed points

l = zeros(length(x),d);

if kernel == 1
    %Gaussian kernel
    kerf = @(z) exp(-z.*z/2)/sqrt(2*pi);
elseif kernel == 2
    %Uniform kernel
    kerf = @(z) 1/2*(abs(z)<=1);
elseif kernel == 3
    %Epanechnikov kernel
    kerf = @(z) 3/4*(1-z^2)*(abs(z)<=1);
end




%Calculate kernel weights before declaring objective function
prod_k = ones(length(x),n);

for i = 1:length(x),
    for j = 1:n,
        for k = 1:d,    %Daisuke: loop to calculate product kernel
            prod_k(i,j) = prod_k(i,j)*kerf((X(j,k)-x(i,k))/h(k));
        end
    end
end


Hmat = eye(n);
%fvec = -repmat(1/n,n,1);
fvec = repmat(0,n,1);
Amat_afriat = [];
 

for i = 1:length(x)
    A = zeros(1+d,1+d);
    for j = 1:n,
         A = A + prod_k(i,j)*[1,X(j,:)-x(i,:)]'*[1,X(j,:)-x(i,:)];
    end
    
    
    for j = 1:n,
        
        B = zeros(1+d,1);       
        B = B + prod_k(i,j)*[1,X(j,:)-x(i,:)]'*y(j);
    
        %Check singularity
        while rcond(A) < 1e-12
            A = A + eye(1+d,1+d).*(1/n);
        end
        
        temp_est = inv(A)*B;
        Aymat(i,j) = n * temp_est(1);
        for k = 1:d
           fp_Aymat(k,j,i) = n * temp_est(1+k); 
        end
    end
end  


%%%This part is only used for the MSE for observed points
%Calculate kernel weights before declaring objective function
prod_k2 = ones(n,n);

for i = 1:n,
    for j = 1:n,
        for k = 1:d,    %Daisuke: loop to calculate product kernel
            prod_k2(i,j) = prod_k2(i,j)*kerf((X(j,k)-X(i,k))/h(k));
        end
    end
end

for i = 1:n
    
    A = zeros(1+d,1+d);
    for j = 1:n,
        A = A + prod_k2(i,j)*[1,X(j,:)-X(i,:)]'*[1,X(j,:)-X(i,:)];
    end
    
    
    for j = 1:n,
        B = zeros(1+d,1);       
        B = B + prod_k2(i,j)*[1,X(j,:)-X(i,:)]'*y(j);
    
        %Check singularity
        while rcond(A) < 1e-12
            A = A + eye(1+d,1+d).*(1/n);
        end
    
        temp_est = inv(A)*B;
        Aymat_obs(i,j) = n * temp_est(1);
        for k = 1:d
           fp_Aymat_obs(k,j,i) = n * temp_est(1+k); 
        end
    end
 end  


%Constrict Amat for quadprog
row = 1;
Amat_afriat=[];
Amat_afriat2=[];
Amat_monotone=[];
for i = 1:length(x)
    for j = 1:length(x)
        if i~=j
            if abs(x(i,:) - x(j,:)) <= dx
                Amat_afriat(row,:) = fp_Aymat(:,:,i)'*(x(j,:)-x(i,:))'-(Aymat(j,:)'-Aymat(i,:)');
                row=row+1;
            end
        end
    end
end

row_mono = 1;
for k = 1:d
    for i = 1:length(x)
        for j = 1:n
            Amat_monotone(row_mono,j) = fp_Aymat(k,j,i);
        end
        row_mono = row_mono+1;
        row = row+1;
    end
end

Amat = [-Amat_monotone;-Amat_afriat];
%bvec = [Amat_monotone*repmat(1/n,n,1);Amat_afriat*repmat(1/n,n,1)];
bvec = [zeros(length(Amat_monotone)+length(Amat_afriat))];
Aeq = ones(1,n);
%beq = 0;
beq = 1;

end