function [MSE_obs,MSE_grd,t1,t2,cnt_dlt,flagBW] = LLKwSCtest(sum_exp,num_var,n,sigma,kernel,h,grid_point,minx_unif,maxx_unif,selection,random,error_model)


%Function of data generation.
if num_var == 1
    fun_data = @(x) x.^sum_exp;                                             %univariate
elseif num_var == 2
    fun_data = @(x) x(:,1).^(sum_exp/2).*x(:,2).^(sum_exp/2);               %bivariate
elseif num_var == 3
    fun_data = @(x) x(:,1).^(sum_exp/3).*x(:,2).^(sum_exp/3).*x(:,3).^(sum_exp/3);
elseif num_var == 4
    fun_data = @(x) x(:,1).^(sum_exp/4).*x(:,2).^(sum_exp/4).*x(:,3).^(sum_exp/4).*x(:,4).^(sum_exp/4);
elseif num_var == 5
    fun_data = @(x) x(:,1).^(sum_exp/5).*x(:,2).^(sum_exp/5).*x(:,3).^(sum_exp/5).*x(:,4).^(sum_exp/5).*x(:,5).^(sum_exp/5);
end



%%Random sample generation
if random == 0 
    rng(19);
end
X=[unifrnd(minx_unif,maxx_unif,n,num_var)];

if error_model == 0
    y=fun_data(X)+normrnd(0,sigma,n,1);
elseif error_model == 1
    y=fun_data(X).*normrnd(1,sigma,n,1);
end

y_true_obs = fun_data(X);



%%Calculate optimal bandwidth by CV and get grid points

tic
warning('off','all')
[h,flagBW] = Opt_BW(X,y,kernel,selection,h);
warning('on','all')
t1=toc;

%%Compute matrix needed for Quadratic Optimization
[Hmat,fvec,Amat,bvec,Aeq,beq,x,dx,row,Aymat,fp_Aymat,Aymat_obs,fp_Aymat_obs,Amat_monotone] = ComputeMatrix(X,y,h,kernel,grid_point);




result=0;   %result = 0: exists some violations
            %result = 1: no violation


v_pt=[];
v_mt = [];    
while result==0

format short g
%[x,phat,what,Amat,row] = ComputeLocalLinear(y,X,Hmat,fvec,Amat,bvec,Aeq,beq,x,v_pt,v_mt,row,Aymat,fp_Aymat,Amat_monotone,Aymat_obs);
[x,phat,Amat,row] = ComputeLocalLinear(y,X,Hmat,fvec,Amat,bvec,Aeq,beq,x,v_pt,v_mt,row,Aymat,fp_Aymat,Amat_monotone,Aymat_obs);



%Check concavity for estimated points by Afriat inequalities

result=1;
cnt_violation = 0;

v_pt = [];
v_mt = [];
cnt = 0;
cnt_mt = 0;
for i = 1:length(x),
    for j = 1:length(x),
        if(i ~=j)
            %if what'*(fp_Aymat(:,:,i)'*(x(i,:)-x(j,:))'-(Aymat(i,:)'-Aymat(j,:)')) > -repmat(1/n,n,1)'*(fp_Aymat(:,:,i)'*(x(i,:)-x(j,:))'-(Aymat(i,:)'-Aymat(j,:)'))+0.05;                
            if phat'*(fp_Aymat(:,:,i)'*(x(i,:)-x(j,:))'-(Aymat(i,:)'-Aymat(j,:)')) > 0.1;                
                result=0;
                cnt=cnt+1;
                cnt_violation=cnt_violation+1;
                v_pt(cnt,:)=[i,j];
            end
        end        
    end
end

if result == 1
    fprintf('Satisfied all Afriat inequalities.\n');
else
    fprintf('Violated %d Afriat inequalities.\n',cnt_violation);
end

end

t2=toc;




%Calculate MSE

y_est_obs = Aymat_obs*phat;
y_est_grd = Aymat*phat;


%Graph obs, estimation for grid, unconstrained estimation for grid%%%%%%%
% y_est_grd_unc = Aymat*repmat(1/n,n,1);
% axis tight; hold on
% plot3(X(:,1),X(:,2),y,'.','MarkerSize',5)
% plot3(X(:,1),X(:,2),y_est_obs,'.','MarkerSize',15)
% plot3(x(:,1),x(:,2),y_est_grd,'.','MarkerSize',15)
% %% plot3(x(:,1),x(:,2),y_est_grd_unc,'.','MarkerSize',15)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


y_true_grd = fun_data(x);

%delete rows which have very negative value
cnt_dlt=0;
for i = 1:length(y_est_grd)
    if y_est_grd(i-cnt_dlt)<-10 
        y_true_grd(i-cnt_dlt)=[];
        y_est_grd(i-cnt_dlt)=[];
        cnt_dlt = cnt_dlt+1;
    end
end

MSE_obs = mean((y_true_obs-y_est_obs).^2);
MSE_grd = mean((y_true_grd-y_est_grd).^2);

fprintf('MSE for observed points:     %d\n',MSE_obs);
fprintf('MSE for equally grid points: %d\n',MSE_grd);

end