 function [h_opt,flag] = Opt_BW(X,y,kernel,selection,h)


n = size(y,1);
% size returns a vector of length 2 with height and width of x
d = size(X,2); 
% Reassign d with the value of d(2) makes d a scalar with the value indicating the 
% number of input variables
flag = 0;

%Small x is point for evaluation
%Capital X is observed points

l = zeros(n,d);

if kernel == 1
    %Gaussian kernel
    kerf = @(z) exp(-z.*z/2)/sqrt(2*pi);

elseif kernel == 2
    %Uniform kernel
    kerf = @(z) 1/2*(abs(z)<=1);

elseif kernel == 3
    %Epanechnikov kernel
    kerf = @(z) 3/4*(1-z^2)*(abs(z)<=1);
end




   %function of leave one out local linear kernel estimator 
    function [MSE] = LOO_CV_LL(h)
        MSE = 0;
        for i = 1:n
            A = zeros(1+d,1+d);
            B = zeros(1+d,1);
            prod_k = ones(n,1);
            for jj = 1:n,
                if jj~=i, 
                    for kk = 1:d,    %Daisuke: loop to calculate product kernel
                       prod_k(jj) = prod_k(jj)*kerf((X(jj,kk)-X(i,kk))/h(kk));
                       %prod_k(jj) = prod_k(jj)*exp(-(X(jj,kk)-X(i,kk))/h_opt(kk).*(X(jj,kk)-X(i,kk))/h_opt(kk)/2)/sqrt(2*pi);
                    end
                end
            end

            for jj = 1:n,
                if jj~=i,
                    A = A + prod_k(jj)*[1,X(jj,:)-X(i,:)]'*[1,X(jj,:)-X(i,:)];
                    B = B + prod_k(jj)*[1,X(jj,:)-X(i,:)]'*y(jj);
                end
            end
            
            while rcond(A) < 1e-12
                A = A + eye(1+d,1+d).*(1/n);
            end
            
            temp_est = inv(A)*B;
            ghat = temp_est(1);
            if isnan(ghat)
            else
                MSE = MSE + (y(i) - ghat)^2;
            end
        end    
    end




 %function of leave one out local linear kernel estimator 
    function [MSE] = LOO_CV_LC(h)
        MSE = 0;
        for i = 1:n
            A = 0;
            B = 0;
            prod_k = ones(n,1);
            for jj = 1:n,
                if jj~=i, 
                    for kk = 1:d,    %Daisuke: loop to calculate product kernel
                       prod_k(jj) = prod_k(jj)*kerf((X(jj,kk)-X(i,kk))/h(kk));
                       %prod_k(jj) = prod_k(jj)*exp(-(X(jj,kk)-X(i,kk))/h_opt(kk).*(X(jj,kk)-X(i,kk))/h_opt(kk)/2)/sqrt(2*pi);
                    end
                end
            end
            
            for jj = 1:n,
                if jj~=i,
                    A = A + prod_k(jj);
                    B = B + prod_k(jj)*y(jj);
                end
            end
            temp_est = B/max(eps,A);
            ghat = temp_est;
            if isnan(ghat)
            else
                MSE = MSE + (y(i) - ghat)^2;
            end
        end    
    end





 %function of 10-fold CV for LL
    function [MSE] = ten_CV_LL(h)
        
        
        MSE = 0;
        for partition = 1:K
            A = zeros(1+d,1+d);
            B = zeros(1+d,1);
            prod_k = ones(sum(Indices~=partition),sum(Indices==partition));
            
 
            X_partition_9 = X(Indices~=partition,:);
            X_partition_1 = X(Indices==partition,:);
            y_partition_9 = y(Indices~=partition);
            y_partition_1 = y(Indices==partition);
            
            
            for ii = 1:sum(Indices==partition),
                for jj = 1:sum(Indices~=partition),
                    for kk = 1:d,    %Daisuke: loop to calculate product kernel
                       prod_k(jj,ii) = prod_k(jj,ii)*kerf((X_partition_9(jj,kk)-X_partition_1(ii,kk))/h(kk));
                       %prod_k(jj) = prod_k(jj)*exp(-(X(jj,kk)-X(i,kk))/h_opt(kk).*(X(jj,kk)-X(i,kk))/h_opt(kk)/2)/sqrt(2*pi);
                    end
                end
            end
            
            
            for ii = 1:sum(Indices==partition),
                for jj = 1:sum(Indices~=partition),
                    A = A + prod_k(jj)*[1,X(jj,:)-X(ii,:)]'*[1,X(jj,:)-X(ii,:)];
                    B = B + prod_k(jj)*[1,X(jj,:)-X(ii,:)]'*y(jj);
                end
                
                while rcond(A) < 1e-12
                    A = A + eye(1+d,1+d).*(1/sum(Indices~=partition));
                end
                
                temp_est = inv(A)*B;
                ghat = temp_est(1);
                if isnan(ghat)
                else
                    MSE = MSE + (y(ii) - ghat)^2;
                end
            end
        end        
    end





 %function of 10-fold CV for LC
    function [MSE] = ten_CV_LC(h)
        MSE = 0;
        for partition = 1:K
            A = zeros(sum(Indices==partition),1);
            B = zeros(sum(Indices==partition),1);
            prod_k = ones(sum(Indices~=partition),sum(Indices==partition));
            X_partition_9 = X(Indices~=partition,:);
            X_partition_1 = X(Indices==partition,:);
            y_partition_9 = y(Indices~=partition);
            y_partition_1 = y(Indices==partition);
            for ii = 1:sum(Indices==partition),
                for jj = 1:sum(Indices~=partition),
                    for kk = 1:d,    %Daisuke: loop to calculate product kernel
                       prod_k(jj,ii) = prod_k(jj,ii)*kerf((X_partition_9(jj,kk)-X_partition_1(ii,kk))/h(kk));
                       %prod_k(jj) = prod_k(jj)*exp(-(X(jj,kk)-X(i,kk))/h_opt(kk).*(X(jj,kk)-X(i,kk))/h_opt(kk)/2)/sqrt(2*pi);
                    end
                end
            end
            
            for ii = 1:sum(Indices==partition),
                for jj = 1:sum(Indices~=partition),
                        A(ii) = A(ii) + prod_k(jj,ii);
                        B(ii) = B(ii) + prod_k(jj,ii)*y_partition_9(jj);
                end

                temp_est(ii) = B(ii)/max(eps,A(ii));
                ghat(ii) = temp_est(ii);
                if isnan(ghat(ii))
                else
                    MSE = MSE + (y_partition_1(ii) - ghat(ii))^2;
                end
            end
        end    
    end



if h == 0,
    if selection == 2,
        h0=ones(d,1);
        
        %options = optimoptions(@fminunc,'Algorithm','quasi-newton');
        options = optimoptions('fminunc','Display','off');
        [h_opt,fval]=fminunc(@LOO_CV_LL,h0,options);
        h_opt
        if all(h_opt >= 10)
            flag=1;
        end
        
        %Setting lower bound of bandwidth
%         for kk = 1:d
%             if h_opt(kk) < 2
%                 h_opt(kk)=2
%             end
%         end
        %h_opt=h_opt.*1.5
    elseif selection == 1,
        h_opt = std(X)*n^(-1/(4+d))
    elseif selection == 3,
        h0=ones(d,1);
        K=10;
        Indices = crossvalind('Kfold', n, K);
        options = optimoptions('fminunc','Display','off');        
        [h_opt,fval]=fminunc(@ten_CV_LL,h0,options);
        h_opt
        if all(h_opt >= 10)
            flag=1;
        end        
        h_opt = h_opt;
    end
else
    h_opt = h
end

end